@@ -1,14 +1,13 @@
 package Catalyst::Model::LDAP;
 
 use strict;
-use base qw/Catalyst::Base/;
+use base qw/Catalyst::Model/;
 use Carp ();
 use NEXT;
 use Net::LDAP;
 
-our $VERSION = '0.07';
-
-__PACKAGE__->mk_accessors('code', 'error');
+our $VERSION = '0.08';
+our $AUTOLOAD;
 
 =head1 NAME
 
@@ -17,10 +16,10 @@ Catalyst::Model::LDAP - LDAP model class for Catalyst
 =head1 SYNOPSIS
 
   # Use the Catalyst helper
-  script/myapp_create.pl model People LDAP ldap.ufl.edu ou=People,dc=ufl,dc=edu
+  script/myapp_create.pl model Person LDAP ldap.ufl.edu ou=People,dc=ufl,dc=edu
 
-  # lib/MyApp/Model/People.pm
-  package MyApp::Model::People;
+  # lib/MyApp/Model/Person.pm
+  package MyApp::Model::Person;
 
   use base 'Catalyst::Model::LDAP';
 
@@ -37,14 +36,19 @@ Catalyst::Model::LDAP - LDAP model class for Catalyst
   1;
 
   # In your controller
-  my $entries = $c->comp('M::People')->search('(sn=TEST)');
-  print $entries->[0]->get_value('sn');
+  my $mesg = $c->model('Person')->search('(sn=TEST)');
+  my @entries = $mesg->entries;
+  print $entries[0]->get_value('sn');
 
 =head1 DESCRIPTION
 
 This is the L<Net::LDAP> model class for Catalyst. It is nothing more
 than a simple wrapper for L<Net::LDAP>.
 
+This class simplifies LDAP searches by letting you configure a base
+DN. Other L<Net::LDAP> methods are supported via Perl's AUTOLOAD
+mechanism.
+
 =head1 METHODS
 
 =head2 new
@@ -62,78 +66,67 @@ sub new {
     return $self;
 }
 
-=head2 search
-
-Search the directory using a given filter. Returns an arrayref
-containing the matching L<Net::LDAP::Entry> objects (if any).
-
-  my $entries = $c->comp('M::People')->search('(sn=TEST)');
-  print $entries->[0]->get_value('sn');
-
-=cut
-
-sub search {
-    my ($self, $filter) = @_;
-
-    my @args = (
-        base   => $self->config->{base},
-        filter => $filter,
-    );
-    my $mesg = $self->_execute('search', @args);
-
-    my @entries = $mesg->entries;
-
-    return \@entries;
-}
-
-=head2 _execute
-
-Bind to the server and execute the specified L<Net::LDAP> method,
-passing in the specified arguments.
-
-=cut
-
-sub _execute {
-    my ($self, $op, @args) = @_;
-
-    my $ldap = $self->_client;
-
-    my $mesg = $ldap->$op(@args, %{ $self->config->{options} });
-    $self->code($mesg->code);
-    $self->error($mesg->error);
-
-    $ldap->unbind;
-
-    return $mesg;
-}
-
 =head2 _client
 
-Return an LDAP connection, bound to the server using the current
-configuration.
+Bind the client using the current configuration and return it.
 
 =cut
 
 sub _client {
     my ($self) = @_;
 
-    my $ldap = Net::LDAP->new(
-        $self->config->{host},
-        %{ $self->config->{options} },
-    ) or Carp::croak $@;
-
     # Default to an anonymous bind
     my @args;
     if (exists $self->config->{dn}) {
         push @args, $self->config->{dn};
-        push @args, password => $self->config->{password} if exists $self->config->{password};
-        push @args, %{ $self->config->{options} }         if exists $self->config->{options};
+        push @args, password => $self->config->{password}
+            if exists $self->config->{password};
+        push @args, %{ $self->config->{options} }
+            if exists $self->config->{options};
     }
 
-    my $mesg = $ldap->bind(@args);
-    Carp::croak 'LDAP error: ' . $mesg->error if $mesg->is_error;
+    my $client = Net::LDAP->new(
+        $self->config->{host},
+        %{ exists $self->config->{options} ? $self->config->{options} : {} },
+    ) or Carp::croak($@);
 
-    return $ldap;
+    my $mesg = $client->bind(@args);
+    Carp::croak('LDAP error: ' . $mesg->error) if $mesg->is_error;
+
+    return $client;
+}
+
+=head2 _execute
+
+Execute the specified LDAP command.
+
+=cut
+
+sub _execute {
+    my ($self, $op, @args) = @_;
+
+    my $client = $self->_client;
+    my $mesg   = $client->$op(@args, %{ $self->config->{options} });
+
+    return $mesg;
+}
+
+# Based on Catalyst::Model::NetBlogger
+sub AUTOLOAD {
+    my ($self, @args) = @_;
+
+    return if $AUTOLOAD =~ /::DESTROY$/;
+    $AUTOLOAD =~ s/^.*:://;
+
+    if ($AUTOLOAD eq 'search' and scalar @args == 1) {
+        # Simplify common case
+        @args = (
+            filter => $args[0],
+            base   => $self->config->{base},
+        );
+    }
+
+    return $self->_execute($AUTOLOAD, @args);
 }
 
 =head1 SEE ALSO
@@ -144,16 +137,6 @@ sub _client {
 
 =item * L<Net::LDAP>
 
-=item * L<Catalyst::Model::LDAP::Cached>
-
-=back
-
-=head1 TODO
-
-=over 4
-
-=item * Add other LDAP methods
-
 =back
 
 =head1 AUTHOR
@@ -165,6 +148,14 @@ mailing list:
 
 L<http://lists.rawmode.org/pipermail/catalyst/2005-June/000712.html>
 
+=head1 ACKNOWLEDGEMENTS
+
+=over 4
+
+=item * Christopher H. Laco, for C<AUTOLOAD> idea
+
+=back
+
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify
